import dbus
import psutil
import os
import subprocess
import time
from musicron.core.plugin import PluginRegistry

class Plugin:
     name = "VLC"

     def __init__(self):
          pass

     def __getProcess(self):
          processName = "vlc"
          for proc in psutil.process_iter():
               if processName.lower() in proc.name().lower():
                    return True
          return False

     def setup(self):
          if not self.__getProcess():
               try:
                    subprocess.Popen(["vlc"])
                    time.sleep(5)
               except FileNotFoundError:
                    print("VLC Not installed")
               
          bus = dbus.SessionBus()
          proxy = bus.get_object('org.mpris.MediaPlayer2.vlc','/org/mpris/MediaPlayer2')
          self.vlc = dbus.Interface(proxy, dbus_interface='org.mpris.MediaPlayer2.Player')
          print("Loaded VLC")
          return self
     
     def Play(self,uri):
          print(uri)
          self.vlc.OpenUri(uri)
          self.vlc.Play()
     
     def Stop(self):
          self.vlc.Stop()






import sys
import os

script_dir = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, os.path.join(script_dir, 'plugins'))

class PluginRegistry:
    def __init__(self):
        self._registry = {}
    
    def register(self,name):
        def decorator(func):
            self._registry[name] = func
            return func
        return decorator

    def setup_plugins(self):
        for name,func in self._registry.items():
            func()


from datetime import datetime, timezone

class TimeDate:
    def __init__(self):
        pass

    def getTime(self,format):
        if format == "UTC":
            now = datetime.now(timezone.utc)
            currentTime = now.strftime("%H:%M")

            return currentTime
        elif format == "local":
            now = datetime.now()
            currentTime = now.strftime("%H:%M")

            return currentTime

    def getDay(self,format):
        if format == "UTC":
            now = datetime.now(timezone.utc)
            currentDay = now.strftime("%a")

            return currentDay
        elif format == "local":
            now = datetime.now()
            currentDay = now.strftime("%a")

            return currentDay

        return day
        




import yaml

class LoadYaml:
    def __init__(self,confFile):

        self.confFile = confFile

        with open(confFile,'r') as file:
            loadedConf = yaml.safe_load(file)
            self.validate(loadedConf)
    
    def validate(self,confFile):

        self.config = []

        for conf in confFile:
            configBuf = { }
            if not conf['name']:
                print("Item requires a name in configuration file")
            else:
                configBuf['name'] = conf['name']
            if not conf['time']:
                print("Item Requires a time")
            else:
                configBuf['time'] = conf['time']
            if not conf['service']:
                print("Item Requires Service")
            else:
                configBuf['service'] = conf['service']

            self.config.append(configBuf)

        return self.config
        
class Config:
    def __init__(self, name: str, time: str, service: dict):
        self.name = name
        self.time = time
        self.service = service

class Service:
    def __init__(self, provider: str, url: str, plugin: str):
        self.provider = provider
        self.url = url
        self.plugin = plugin

class ConfigFile:

    def __init__(self, configFile):
        self.configs = []

        try:
            loadedConfig = LoadYaml(configFile) # Load YAML and begin validation
        except FileNotFoundError:
            print("Configuration file not found")

        for item in loadedConfig.config:
            service_data = item['service']
            service = Service(provider=service_data['provider'],url=service_data['url'],plugin=service_data['plugin'])
            config = Config(name=item['name'], time=item['time'], service=service)
            self.configs.append(config)
                    
    def get_configs(self):
        return self.configs


#!/usr/bin/env python

from datetime import datetime
from dateutil import tz
from pathlib import Path
from musicron.core.config import ConfigFile,LoadYaml
from musicron.core.timedate import TimeDate
from musicron.core.thread import PluginThread
from musicron.core.plugin import PluginRegistry

import importlib2
import sys
import functools
import os
import time

class MusiCron:
    def __init__(self, plugins:list=[]):

        self.player_thread = None

        self.plugins = PluginRegistry()
        for plugin in plugins:
            imported_module = importlib2.import_module('musicron.plugins.' + plugin)
            imported_instance = imported_module.Plugin()
            self.plugins.register(imported_module.Plugin.name)(imported_instance.setup())

    def play(self,provider,URL):
        self.player_thread = PluginThread(self.plugins,provider,URL)
        self.player_thread.start()

    def stop(self,provider,URL):
        self.player_thread.stop()
        
    def run(self):
        configPath = os.path.expanduser('~/.musicron.conf')  
        try:
            if os.path.isfile(configPath):
                configPath = os.path.expanduser('~/.musicron.conf')

                config_file = ConfigFile(configPath) #start reading the config YAML file
                configs = config_file.get_configs()
            else:
                configPath = 'musicron.conf.sample'
        except FileNotFoundError:
            print("Config File Not Found")

        while(True):
            timeDate = TimeDate()
            
            currentTime = timeDate.getTime("local")
            currentDay = timeDate.getDay("local")
            currentNow = currentTime + " " + currentDay
            
            currentTimeUTC = timeDate.getTime("UTC")
            currentDayUTC = timeDate.getDay("UTC")

            currentNowUTC = "U" + currentTimeUTC + " " + currentDayUTC

            for config in configs:
                name = config.name
                playTime = config.time
                provider = config.service.provider
                plugin = config.service.plugin
                URL = config.service.url

                if playTime.find("-") != -1:
                    runTime = playTime.split("-")
                    startTime = runTime[0]
                    endTime = runTime[1]
                    print(startTime)
                    print(endTime)
                else:
                    startTime = playTime

                if startTime == currentTime or startTime == currentNow or startTime == currentTimeUTC or startTime == currentNowUTC:
                    print("Playing " + name + "at" + startTime)
                    self.play(provider,URL)
                elif endTime == currentTime or endTime == currentNow or endTime == currentTimeUTC or endTime == currentNowUTC:
                    print("Playing " + name + "at" + startTime)
                    self.stop(provider,URL)
                else:
                    pass

            time.sleep(60)

if __name__ == "__main__":
    path = os.path.join(os.path.dirname(__file__), 'plugins')
    if os.path.isdir(path):
        dir_list = os.listdir(path)
        app = MusiCron(dir_list)
        app.run()
    else:
        print("Plugins directory not found.")



from setuptools import setup, find_packages, find_namespace_packages

setup(
    name="musicron",
    version="0.1.14",
    author="Wesley Fisher",
    author_email="wfisher@praetor.tel",
    description="A cron for music!",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/gwfisher/musicron",
    packages=find_namespace_packages(),
    exclude_package_data={
        'musicron': ['plugins/__init__.py']
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
    install_requires=[
        "pyyaml",
        "dbus-python",
        "pathlib",
        "importlib2",
        "psutil",
        "python-dateutil"
    ],
)



import threading

class PluginThread(threading.Thread):

    def __init__(self,plugins,provider,URL):
        super().__init__()
        self.provider = provider
        self.URL = URL
        self.plugins = plugins

    def run(self):
        self.active_plugin = self.plugins._registry.get(self.provider)
        self.active_plugin.Play(self.URL)
    
    def stop(self):
        self.active_plugin.Stop()